import "../../chunks/chunk-U5RRZUYZ.js";

// src/backend/api/GithubApi.ts
import { HttpError } from "alinea/core/HttpError";
import { base64, btoa } from "alinea/core/util/Encoding";
import { join } from "alinea/core/util/Paths";
import { ChangeType } from "../data/ChangeSet.js";
import { applyJsonPatch } from "../util/JsonPatch.js";
function githubApi(options) {
  const target = {
    async mutate(ctx, params) {
      let commitMessage = "Alinea content update";
      if (ctx.user.email && ctx.user.name)
        commitMessage += `
Co-authored-by: ${ctx.user.name} <${ctx.user.email}>`;
      return {
        commitHash: await applyChangesToRepo(
          options,
          params.mutations.flatMap((m) => {
            return m.changes;
          }),
          commitMessage
        )
      };
    }
  };
  const history = {
    async list(ctx, file) {
      return getFileCommitHistory(options, file);
    },
    async revision(ctx, file, ref) {
      const content = await getFileContentAtCommit(options, file, ref);
      try {
        return content ? JSON.parse(content) : void 0;
      } catch (error) {
        return void 0;
      }
    }
  };
  return { target, history };
}
function graphQL(query, variables, token) {
  return fetch("https://api.github.com/graphql", {
    method: "POST",
    headers: {
      Authorization: `Bearer ${token}`,
      "Content-Type": "application/json"
    },
    body: JSON.stringify({ query, variables })
  }).then(async (response) => {
    if (response.ok)
      return response.json();
    throw new HttpError(response.status, await response.text());
  });
}
async function getFileCommitHistory({ owner, repo, branch, authToken, rootDir }, file) {
  const result = await graphQL(
    `query GetFileHistory($owner: String!, $repo: String!, $branch: String!, $path: String!) {
      repository(owner: $owner, name: $repo) {
        ref(qualifiedName: $branch) {
          target {
            ... on Commit {
              history(path: $path, first: 100) {
                nodes {
                  oid
                  committedDate
                  message
                  author {
                    name
                    email
                  }
                }
              }
            }
          }
        }
      }
    }`,
    { owner, repo, branch, path: join(rootDir, file) },
    authToken
  );
  const commits = result.data.repository.ref.target.history.nodes;
  return commits.map((commit) => ({
    ref: commit.oid,
    createdAt: new Date(commit.committedDate).getTime(),
    file,
    user: commit.author ? { name: commit.author.name, email: commit.author.email } : void 0,
    description: commit.message
  }));
}
async function getFileContentAtCommit({ owner, repo, authToken, rootDir }, file, ref) {
  const result = await graphQL(
    `query GetFileContent($owner: String!, $repo: String!, $expression: String!) {
      repository(owner: $owner, name: $repo) {
        object(expression: $expression) {
          ... on Blob {
            text
          }
        }
      }
    }`,
    { owner, repo, expression: `${ref}:${join(rootDir, file)}` },
    authToken
  );
  return result.data.repository.object?.text;
}
async function applyChangesToRepo(options, changes, commitMessage) {
  const { additions, deletions } = await processChanges(options, changes);
  const { owner, repo, branch, authToken } = options;
  return graphQL(
    `mutation CreateCommitOnBranch($input: CreateCommitOnBranchInput!) {
      createCommitOnBranch(input: $input) {
        commit {
          oid
        }
      }
    }`,
    {
      input: {
        branch: {
          repositoryNameWithOwner: `${owner}/${repo}`,
          branchName: branch
        },
        message: {
          headline: commitMessage
        },
        fileChanges: {
          additions,
          deletions
        },
        expectedHeadOid: await getLatestCommitOid(options)
      }
    },
    authToken
  ).then((result) => result.data.createCommitOnBranch.commit.oid);
}
async function processChanges(options, changes) {
  const additions = [];
  const deletions = [];
  for (const change of changes) {
    switch (change.type) {
      case ChangeType.Write:
      case ChangeType.Upload: {
        const file = join(options.rootDir, change.file);
        additions.push({
          path: file,
          contents: change.type === ChangeType.Write ? btoa(change.contents) : await fetchUploadedContent(change.url)
        });
        break;
      }
      case ChangeType.Delete: {
        const file = join(options.rootDir, change.file);
        deletions.push({ path: file });
        break;
      }
      case ChangeType.Rename: {
        const from = join(options.rootDir, change.from);
        const to = join(options.rootDir, change.to);
        const isFolder = await isPathFolder(options, from);
        if (isFolder) {
          const files = await listFilesInFolder(options, from);
          const contents = await Promise.all(
            files.map((file) => fetchFileContent(options, file))
          );
          for (let i = 0; i < files.length; i++) {
            const file = files[i];
            deletions.push({ path: file });
            const path = to + file.slice(from.length);
            additions.push({
              path,
              contents: contents[i]
            });
          }
        } else {
          deletions.push({ path: from });
          additions.push({
            path: join(options.rootDir, to),
            contents: await fetchFileContent(options, from)
          });
        }
        break;
      }
      case ChangeType.Patch: {
        const file = join(options.rootDir, change.file);
        const currentContent = await fetchFileContent(options, file);
        const patchedContent = applyPatch(currentContent, change.patch);
        additions.push({
          path: file,
          contents: btoa(patchedContent)
        });
        break;
      }
    }
  }
  return { additions, deletions };
}
async function getLatestCommitOid({
  owner,
  repo,
  branch,
  authToken
}) {
  return graphQL(
    `query GetLatestCommit($owner: String!, $repo: String!, $branch: String!) {
      repository(owner: $owner, name: $repo) {
        ref(qualifiedName: $branch) {
          target {
            oid
          }
        }
      }
    }`,
    { owner, repo, branch },
    authToken
  ).then((result) => result.data.repository.ref.target.oid);
}
async function isPathFolder({ owner, repo, branch, authToken }, path) {
  return graphQL(
    `query IsPathFolder($owner: String!, $repo: String!, $expression: String!) {
      repository(owner: $owner, name: $repo) {
        object(expression: $expression) {
          ... on Tree {
            entries {
              name
            }
          }
        }
      }
    }
  `,
    { owner, repo, expression: `${branch}:${path}` },
    authToken
  ).then((result) => result.data.repository.object !== null);
}
async function listFilesInFolder({ owner, repo, branch, authToken }, path) {
  return graphQL(
    `query ListFiles($owner: String!, $repo: String!, $expression: String!) {
      repository(owner: $owner, name: $repo) {
        object(expression: $expression) {
          ... on Tree {
            entries {
              name
              type
              object {
                ... on Blob {
                  oid
                }
                ... on Tree {
                  entries {
                    name
                    type
                    object {
                      ... on Blob {
                        oid
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  `,
    { owner, repo, expression: `${branch}:${path}` },
    authToken
  ).then((result) => flattenFileList(result.data.repository.object.entries, path));
}
function flattenFileList(entries, basePath) {
  let files = [];
  for (const entry of entries) {
    if (entry.type === "blob") {
      files.push(`${basePath}/${entry.name}`);
    } else if (entry.type === "tree") {
      files = files.concat(
        flattenFileList(entry.object.entries, `${basePath}/${entry.name}`)
      );
    }
  }
  return files;
}
async function fetchFileContent({ owner, repo, branch, authToken }, path) {
  return graphQL(
    `query GetFileContent($owner: String!, $repo: String!, $expression: String!) {
      repository(owner: $owner, name: $repo) {
        object(expression: $expression) {
          ... on Blob {
            text
          }
        }
      }
    }
  `,
    { owner, repo, expression: `${branch}:${path}` },
    authToken
  ).then((result) => result.data.repository.object.text);
}
async function fetchUploadedContent(url) {
  const response = await fetch(url);
  return base64.stringify(new Uint8Array(await response.arrayBuffer()));
}
function applyPatch(content, patch) {
  try {
    const jsonContent = JSON.parse(content);
    applyJsonPatch(jsonContent, patch);
    return JSON.stringify(jsonContent, null, 2);
  } catch (error) {
    console.error("Error applying patch:", error);
    return content;
  }
}
export {
  githubApi
};
