import {
  Builder,
  QueryBatch
} from "./chunk-OMOA6SIP.js";
import {
  eq
} from "./chunk-PE5G5FUO.js";
import {
  column
} from "./chunk-GR7UQZEH.js";
import {
  callFunction,
  jsonAggregateArray,
  jsonArray,
  table
} from "./chunk-AOFVF5VC.js";
import {
  Sql,
  getData,
  getQuery,
  getResolver,
  getSelection,
  getSql,
  getTable,
  getTarget,
  hasQuery,
  hasSelection,
  internalResolver,
  sql
} from "./chunk-NXCD6SNE.js";

// node_modules/rado/dist/universal/transactions.js
function txGenerator(create) {
  function run(tx) {
    const iter = create(tx);
    const next = (inner) => {
      if (inner instanceof Promise)
        return inner.then(iter.next.bind(iter)).catch(iter.throw.bind(iter)).then(handle);
      try {
        return handle(iter.next(inner));
      } catch (err) {
        return handle(iter.throw(err));
      }
    };
    const handle = ({ done, value }) => {
      if (done)
        return value;
      return next(typeof value === "function" ? tx.transaction(value) : value);
    };
    return next();
  }
  return Object.assign(run, {
    *[Symbol.iterator]() {
      return yield run;
    }
  });
}

// node_modules/rado/dist/core/Resolver.js
var Resolver = class {
  #driver;
  #dialect;
  constructor(driver, dialect) {
    this.#driver = driver;
    this.#dialect = dialect;
  }
  toSQL(query) {
    const emitter = this.#dialect.emit(query);
    return { sql: emitter.sql, params: emitter.bind() };
  }
  prepare(query, name) {
    const isSelection = hasSelection(query);
    const mapRow = isSelection ? getSelection(query).mapRow : void 0;
    const emitter = this.#dialect.emit(query);
    const stmt = this.#driver.prepare(emitter.sql, {
      isSelection,
      name
    });
    return new PreparedStatement(emitter, stmt, mapRow, this.#driver);
  }
  batch(queries) {
    return new Batch(
      this.#driver,
      queries.map((query) => {
        const isSelection = hasSelection(query);
        const mapRow = isSelection ? getSelection(query).mapRow : void 0;
        const emitter = this.#dialect.emit(query);
        return { sql: emitter.sql, params: emitter.bind(), isSelection, mapRow };
      })
    );
  }
};
var Batch = class {
  #driver;
  #queries;
  constructor(driver, queries) {
    this.#driver = driver;
    this.#queries = queries;
  }
  #transform = (results) => {
    const ctx = {
      values: void 0,
      index: 0,
      specs: this.#driver
    };
    for (let i = 0; i < this.#queries.length; i++) {
      const { mapRow } = this.#queries[i];
      if (!mapRow)
        continue;
      const rows = results[i];
      for (let j = 0; j < results[i].length; j++) {
        ctx.values = rows[i];
        ctx.index = 0;
        rows[i] = mapRow(ctx);
      }
    }
    return results;
  };
  execute() {
    const results = this.#driver.batch(this.#queries);
    if (results instanceof Promise)
      return results.then(this.#transform);
    return this.#transform(results);
  }
};
var PreparedStatement = class {
  #emitter;
  #stmt;
  #mapRow;
  #specs;
  constructor(emitter, stmt, mapRow, specs) {
    this.#emitter = emitter;
    this.#stmt = stmt;
    this.#mapRow = mapRow;
    this.#specs = specs;
  }
  #transform = (rows) => {
    if (!this.#mapRow)
      return rows;
    const ctx = {
      values: void 0,
      index: 0,
      specs: this.#specs
    };
    for (let i = 0; i < rows.length; i++) {
      ctx.values = rows[i];
      ctx.index = 0;
      rows[i] = this.#mapRow(ctx);
    }
    return rows;
  };
  all(inputs) {
    const rows = this.#stmt.values(this.#emitter.bind(inputs));
    if (rows instanceof Promise)
      return rows.then(this.#transform);
    return this.#transform(rows);
  }
  get(inputs) {
    const rows = this.all(inputs);
    if (rows instanceof Promise)
      return rows.then((rows2) => rows2[0]);
    return rows[0];
  }
  run(inputs) {
    return this.#stmt.run(this.#emitter.bind(inputs));
  }
  async execute(inputs) {
    return this.all(inputs);
  }
  free() {
    this.#stmt.free();
  }
};

// node_modules/rado/dist/core/Database.js
var Database = class extends Builder {
  driver;
  dialect;
  diff;
  [internalResolver];
  constructor(driver, dialect, diff) {
    const resolver = new Resolver(driver, dialect);
    super({ resolver });
    this[internalResolver] = resolver;
    this.driver = driver;
    this.dialect = dialect;
    this.diff = diff;
  }
  close() {
    return this.driver.close();
  }
  [Symbol.dispose]() {
    this.close();
  }
  async [Symbol.asyncDispose]() {
    return this.close();
  }
  create(...tables) {
    return new QueryBatch(
      getResolver(this),
      tables.flatMap((table2) => getTable(table2).create())
    );
  }
  drop(...tables) {
    return new QueryBatch(
      getResolver(this),
      tables.map((table2) => getTable(table2).drop())
    );
  }
  migrate(...tables) {
    const computeDiff = this.diff;
    return this.transaction(
      txGenerator(function* (tx) {
        for (const table2 of tables) {
          const diff = yield* computeDiff(table2);
          if (diff.length > 0)
            yield* tx.batch(diff.map(sql.unsafe));
        }
      })
    );
  }
  batch(queries) {
    return new QueryBatch(getResolver(this), queries);
  }
  execute(input) {
    const emitter = this.dialect.emit(input);
    if (emitter.hasParams)
      throw new Error("Query has parameters");
    return this.driver.exec(emitter.sql);
  }
  transaction(run, options = {}) {
    return this.driver.transaction(
      (inner) => {
        const tx = new Transaction(inner, this.dialect, this.diff);
        return run(tx);
      },
      { async: run.constructor.name === "AsyncFunction", ...options }
    );
  }
};
var Rollback = class extends Error {
  constructor(data) {
    super("Rollback");
    this.data = data;
  }
};
var Transaction = class extends Database {
  rollback(data) {
    throw new Rollback(data);
  }
};
var SyncDatabase = class extends Database {
};
var AsyncDatabase = class extends Database {
};

// node_modules/rado/dist/sqlite/columns.js
function boolean(name) {
  return column({
    name,
    type: column.integer(),
    mapFromDriverValue(value) {
      return value === 1;
    },
    mapToDriverValue(value) {
      return value ? 1 : 0;
    }
  });
}
function integer(name) {
  return column({ name, type: column.integer() });
}
function text(name) {
  return column({ name, type: column.text() });
}

// node_modules/rado/dist/core/Dialect.js
var Dialect = class {
  runtime;
  #createEmitter;
  constructor(runtime, createEmitter) {
    this.runtime = runtime;
    this.#createEmitter = createEmitter;
  }
  emit = (input) => {
    const sql2 = hasQuery(input) ? getQuery(input) : getSql(input);
    const emitter = new this.#createEmitter(this.runtime);
    sql2.emitTo(emitter);
    return emitter;
  };
  inline = (input) => {
    const sql2 = hasQuery(input) ? getQuery(input) : getSql(input);
    const emitter = new this.#createEmitter(this.runtime);
    sql2.inlineValues().emitTo(emitter);
    return emitter.sql;
  };
};

// node_modules/rado/dist/core/Param.js
var NamedParam = class {
  constructor(name) {
    this.name = name;
  }
};
var ValueParam = class {
  constructor(value) {
    this.value = value;
  }
};

// node_modules/rado/dist/core/Emitter.js
var Emitter = class {
  #runtime;
  sql = "";
  params = [];
  constructor(runtime) {
    this.#runtime = runtime;
  }
  get hasParams() {
    return this.params.length > 0;
  }
  bind(inputs) {
    return this.params.map((param) => {
      if (param instanceof ValueParam)
        return this.processValue(param.value);
      if (inputs && param.name in inputs)
        return this.processValue(inputs[param.name]);
      throw new Error(`Missing input for named parameter: ${param.name}`);
    });
  }
  processValue(value) {
    return value;
  }
  emitIdentifierOrSelf(value) {
    if (value === Sql.SELF_TARGET) {
      if (!this.selfName)
        throw new Error("Self target not defined");
      this.emitIdentifier(this.selfName);
    } else {
      this.emitIdentifier(value);
    }
  }
  selfName;
  emitSelf({ name, inner }) {
    this.selfName = name;
    inner.emitTo(this);
    this.selfName = void 0;
  }
  emitUnsafe(value) {
    this.sql += value;
  }
  emitField({ targetName, fieldName }) {
    this.emitIdentifierOrSelf(targetName);
    this.emitUnsafe(".");
    this.emitIdentifier(fieldName);
  }
  emitCreateTable(tableApi) {
    sql.join([
      sql`create table`,
      //ifNotExists ? sql`if not exists` : undefined,
      tableApi.target(),
      sql`(${tableApi.createDefinition()})`
    ]).emitTo(this);
  }
  emitColumn(column2) {
    sql.join([
      column2.type,
      column2.primary && sql`primary key`,
      column2.notNull && sql`not null`,
      column2.isUnique && sql`unique`,
      column2.autoIncrement && sql`autoincrement`,
      column2.defaultValue && sql`default ${column2.defaultValue}`,
      column2.references && sql`references ${sql.chunk("emitReferences", [column2.references()])}`,
      column2.onUpdate && sql`on update ${column2.onUpdate}`
    ]).emitTo(this);
  }
  emitReferences(fields) {
    callFunction(
      sql.identifier(fields[0].targetName),
      fields.map((field) => sql.identifier(field.fieldName))
    ).emitTo(this);
  }
  emitDelete(deleteOp) {
    const { cte, from, where, returning } = getData(deleteOp);
    if (cte)
      this.emitWith(cte);
    const table2 = getTable(from);
    sql.query({
      deleteFrom: sql.identifier(table2.name),
      where,
      returning
    }).emitTo(this);
  }
  emitInsert(insert) {
    const {
      cte,
      into,
      values,
      select,
      onConflict,
      onDuplicateKeyUpdate,
      returning
    } = getData(insert);
    if (cte)
      this.emitWith(cte);
    const table2 = getTable(into);
    const tableName = sql.identifier(table2.name);
    sql.query({
      insertInto: sql`${tableName}(${table2.listColumns()})`,
      ...values ? { values } : { "": select },
      onConflict,
      onDuplicateKeyUpdate,
      returning
    }).inlineFields(false).emitTo(this);
  }
  emitSelect({
    select,
    cte,
    from,
    distinct,
    distinctOn,
    where,
    groupBy,
    orderBy,
    having,
    limit,
    offset
  }) {
    if (cte)
      this.emitWith(cte);
    const prefix = distinctOn ? sql`distinct on (${sql.join(distinctOn, sql`, `)})` : distinct && sql`distinct`;
    sql.query({
      select: sql.join([prefix, select]),
      from,
      where,
      groupBy,
      orderBy,
      having,
      limit,
      offset
    }).emitTo(this);
  }
  emitUnion({ left, operator, right }) {
    sql.join([getQuery(left), operator, getQuery(right)]).emitTo(this);
  }
  emitUpdate(update) {
    const { cte, table: table2, set, where, returning } = getData(update);
    const tableApi = getTable(table2);
    if (cte)
      this.emitWith(cte);
    sql.query({
      update: sql.identifier(tableApi.name),
      set,
      where,
      returning
    }).inlineFields(false).emitTo(this);
  }
  emitWith(cte) {
    sql.query({
      [cte.recursive ? "withRecursive" : "with"]: sql.join(
        cte.definitions.map((cte2) => {
          const query = getQuery(cte2);
          const target = getTarget(cte2);
          return sql`${target} as (${query})`;
        }),
        sql`, `
      )
    }).add(sql` `).emitTo(this);
  }
  emitInclude(data) {
    const wrapQuery = Boolean(data.limit || data.offset || data.orderBy);
    const innerQuery = sql.chunk("emitSelect", data);
    const inner = wrapQuery ? sql`select * from (${innerQuery})` : innerQuery;
    if (!data.select)
      throw new Error("No selection defined");
    const fields = data.select.fieldNames();
    const subject = jsonArray(
      ...fields.map((name) => sql`_.${sql.identifier(name)}`)
    );
    sql`(select ${data.first ? subject : jsonAggregateArray(subject)} from (${inner}) as _)`.emitTo(this);
  }
  emitUniversal(runtimes) {
    const sql2 = runtimes[this.#runtime] ?? runtimes.default;
    if (!sql2)
      throw new Error("Unsupported runtime");
    sql2.emitTo(this);
  }
};

// node_modules/rado/dist/sqlite/dialect.js
var DOUBLE_QUOTE = '"';
var ESCAPE_DOUBLE_QUOTE = '""';
var MATCH_DOUBLE_QUOTE = /"/g;
var SINGLE_QUOTE = "'";
var ESCAPE_SINGLE_QUOTE = "''";
var MATCH_SINGLE_QUOTE = /'/g;
var sqliteDialect = new Dialect(
  "sqlite",
  class extends Emitter {
    processValue(value) {
      return typeof value === "boolean" ? value ? 1 : 0 : value;
    }
    emitValue(value) {
      this.sql += "?";
      this.params.push(new ValueParam(value));
    }
    emitJsonPath({ target, asSql, segments }) {
      target.emitTo(this);
      this.sql += asSql ? "->>" : "->";
      this.sql += this.quoteString(
        `$${segments.map((p) => typeof p === "number" ? `[${p}]` : `.${p}`).join("")}`
      );
    }
    emitInline(value) {
      if (value === null || value === void 0)
        return this.sql += "null";
      if (typeof value === "number")
        return this.sql += value;
      if (typeof value === "string")
        return this.sql += this.quoteString(value);
      if (typeof value === "boolean")
        return this.sql += value ? "1" : "0";
      this.sql += `json(${this.quoteString(JSON.stringify(value))})`;
    }
    emitPlaceholder(name) {
      this.sql += "?";
      this.params.push(new NamedParam(name));
    }
    emitIdentifier(identifier) {
      this.sql += DOUBLE_QUOTE + identifier.replace(MATCH_DOUBLE_QUOTE, ESCAPE_DOUBLE_QUOTE) + DOUBLE_QUOTE;
    }
    quoteString(input) {
      return SINGLE_QUOTE + input.replace(MATCH_SINGLE_QUOTE, ESCAPE_SINGLE_QUOTE) + SINGLE_QUOTE;
    }
  }
);

// node_modules/rado/dist/sqlite/diff.js
var TableInfo = table("TableInfo", {
  cid: integer().notNull(),
  name: text().notNull(),
  type: text().notNull(),
  notnull: boolean().notNull(),
  dflt_value: text(),
  pk: integer().notNull()
});
var SqliteMaster = table("SqliteMaster", {
  type: text().notNull(),
  name: text().notNull(),
  tbl_name: text().notNull(),
  rootpage: integer().notNull(),
  sql: text().notNull()
});
var inline = (sql2) => sqliteDialect.inline(sql2);
var sqliteDiff = (hasTable) => {
  return txGenerator(function* (tx) {
    const tableApi = getTable(hasTable);
    const columnInfo = yield* tx.select(TableInfo).from(sql`pragma_table_info(${sql.inline(tableApi.name)}) as "TableInfo"`);
    const indexInfo = yield* tx.select(SqliteMaster).from(sql`sqlite_master as "SqliteMaster"`).where(
      eq(SqliteMaster.tbl_name, tableApi.name),
      eq(SqliteMaster.type, "index")
    );
    const hasSinglePrimaryKey = columnInfo.reduce((acc, column2) => acc + column2.pk, 0) === 1;
    const localColumns = new Map(
      columnInfo.map((column2) => {
        return [
          column2.name,
          inline(
            sql.chunk("emitColumn", {
              type: sql.unsafe(column2.type.toLowerCase()),
              notNull: column2.notnull,
              primary: hasSinglePrimaryKey && column2.pk === 1,
              defaultValue: column2.dflt_value !== null ? sql.unsafe(column2.dflt_value) : void 0
            })
          )
        ];
      })
    );
    const schemaColumns = new Map(
      Object.entries(tableApi.columns).map(([name, column2]) => {
        const columnApi = getData(column2);
        return [
          columnApi.name ?? name,
          inline(sql.chunk("emitColumn", columnApi))
        ];
      })
    );
    const localIndexes = new Map(
      indexInfo.map((index) => [index.name, index.sql])
    );
    const schemaIndexes = new Map(
      Object.entries(tableApi.indexes()).map(([name, index]) => {
        const indexApi = getData(index);
        return [name, inline(indexApi.toSql(tableApi.name, name, false))];
      })
    );
    const stmts = [];
    const columnNames = /* @__PURE__ */ new Set([
      ...localColumns.keys(),
      ...schemaColumns.keys()
    ]);
    for (const columnName of columnNames) {
      const localInstruction = localColumns.get(columnName);
      const schemaInstruction = schemaColumns.get(columnName);
      if (!schemaInstruction) {
        stmts.push(
          sql.query({
            alterTable: sql.identifier(tableApi.name),
            dropColumn: sql.identifier(columnName)
          })
        );
      } else if (!localInstruction) {
        stmts.push(
          sql.query({
            alterTable: sql.identifier(tableApi.name),
            addColumn: [
              sql.identifier(columnName),
              sql.unsafe(schemaInstruction)
            ]
          })
        );
      } else if (localInstruction !== schemaInstruction) {
        return recreate();
      }
    }
    try {
      yield* txGenerator(function* (sp) {
        yield* sp.batch(stmts);
        const [tableInfo] = yield* sp.select(SqliteMaster.sql).from(sql`sqlite_master as "SqliteMaster"`).where(
          eq(SqliteMaster.tbl_name, tableApi.name),
          eq(SqliteMaster.type, "table")
        );
        sp.rollback(tableInfo);
      });
    } catch (err) {
      if (!(err instanceof Rollback))
        throw err;
      const localInstruction = err.data;
      const schemaInstruction = inline(tableApi.createTable());
      const stripStmt = (q) => q.slice("create table ".length);
      if (stripStmt(localInstruction) !== stripStmt(schemaInstruction)) {
        return recreate();
      }
    }
    const indexNames = /* @__PURE__ */ new Set([
      ...localIndexes.keys(),
      ...schemaIndexes.keys()
    ]);
    for (const indexName of indexNames) {
      const localInstruction = localIndexes.get(indexName);
      const schemaInstruction = schemaIndexes.get(indexName);
      const dropLocal = sql.query({
        dropIndex: sql.identifier(indexName),
        on: sql.identifier(tableApi.name)
      });
      if (!schemaInstruction) {
        stmts.unshift(dropLocal);
      } else if (!localInstruction) {
        stmts.push(sql.unsafe(schemaInstruction));
      } else if (localInstruction !== schemaInstruction) {
        stmts.unshift(dropLocal);
        stmts.push(sql.unsafe(schemaInstruction));
      }
    }
    return stmts.map(inline);
    function recreate() {
      const tempName = `new_${tableApi.name}`;
      const tempTable = table(tempName, tableApi.columns);
      const missingColumns = Array.from(columnNames).filter(
        (name) => !localColumns.has(name)
      );
      const selection = { ...hasTable };
      for (const name of missingColumns) {
        selection[name] = getData(tableApi.columns[name]).defaultValue ?? sql`null`;
      }
      return [
        // Create a new temporary table with the new definition
        tableApi.createTable(tempName),
        // Copy the data from the old table to the new table
        getQuery(
          tx.insert(tempTable).select(tx.select(selection).from(hasTable))
        ),
        // Drop the old table
        sql.query({ dropTable: sql.identifier(tableApi.name) }),
        // Rename the temporary table to the old table name
        sql.query({
          alterTable: sql.identifier(tempName),
          renameTo: sql.identifier(tableApi.name)
        }),
        // Create missing indexes
        ...tableApi.createIndexes()
      ].map(inline);
    }
  });
};

// node_modules/rado/dist/sqlite/transactions.js
var locks = /* @__PURE__ */ new WeakMap();
function execTransaction(driver, depth, wrap, run, options) {
  const needsLock = options.async;
  const behavior = options.behavior ?? "deferred";
  if (!needsLock)
    return transact();
  let trigger;
  const lock = new Promise((resolve) => trigger = resolve);
  const current = Promise.resolve(locks.get(driver));
  locks.set(driver, lock);
  return current.then(transact).finally(trigger);
  function transact() {
    try {
      driver.exec(depth > 0 ? `savepoint d${depth}` : `begin ${behavior}`);
      const result = run(wrap(depth + 1));
      if (result instanceof Promise)
        return result.then(release, rollback);
      return release(result);
    } catch (error) {
      return rollback(error);
    }
  }
  function release(result) {
    driver.exec(depth > 0 ? `release d${depth}` : "commit");
    return result;
  }
  function rollback(error) {
    driver.exec(depth > 0 ? `rollback to d${depth}` : "rollback");
    throw error;
  }
}

// node_modules/rado/dist/driver/sql.js.js
var PreparedStatement2 = class {
  constructor(client, stmt) {
    this.client = client;
    this.stmt = stmt;
  }
  *iterate(params) {
    this.stmt.bind(params);
    while (this.stmt.step())
      yield this.stmt.getAsObject();
    this.stmt.reset();
  }
  *iterateValues(params) {
    this.stmt.bind(params);
    while (this.stmt.step())
      yield this.stmt.get();
    this.stmt.reset();
  }
  all(params) {
    return Array.from(this.iterate(params));
  }
  run(params) {
    this.stmt.run(params);
    this.stmt.reset();
    return { rowsAffected: this.client.getRowsModified() };
  }
  get(params) {
    return this.all(params)[0] ?? null;
  }
  values(params) {
    return Array.from(this.iterateValues(params));
  }
  free() {
    this.stmt.free();
  }
};
var SqlJsDriver = class _SqlJsDriver {
  constructor(client, depth = 0) {
    this.client = client;
    this.depth = depth;
  }
  parsesJson = false;
  supportsTransactions = true;
  exec(query) {
    this.client.exec(query);
  }
  close() {
    this.client.close();
  }
  prepare(sql2) {
    return new PreparedStatement2(this.client, this.client.prepare(sql2));
  }
  batch(queries) {
    return this.transaction(
      (tx) => queries.map(({ sql: sql2, params }) => tx.prepare(sql2).values(params)),
      {}
    );
  }
  transaction(run, options) {
    return execTransaction(
      this,
      this.depth,
      (depth) => new _SqlJsDriver(this.client, depth),
      run,
      options
    );
  }
};
function connect(db) {
  return new SyncDatabase(new SqlJsDriver(db), sqliteDialect, sqliteDiff);
}

export {
  txGenerator,
  SyncDatabase,
  AsyncDatabase,
  Dialect,
  NamedParam,
  ValueParam,
  Emitter,
  sqliteDialect,
  sqliteDiff,
  execTransaction,
  connect
};
