import {
  package_default
} from "../chunks/chunk-7EETFPPX.js";
import {
  PLazy
} from "../chunks/chunk-IKINPSS5.js";
import "../chunks/chunk-U5RRZUYZ.js";

// src/cloud/CloudBackend.ts
import { Response } from "@alinea/iso";
import { AuthAction } from "alinea/backend/Auth";
import { router } from "alinea/backend/router/Router";
import { HttpError } from "alinea/core/HttpError";
import { outcome, Outcome } from "alinea/core/Outcome";
import { base64 } from "alinea/core/util/Encoding";
import { verify } from "alinea/core/util/JWT";
import { Workspace } from "alinea/core/Workspace";
import { AuthResultType } from "./AuthResult.js";
import { cloudConfig } from "./CloudConfig.js";
var RemoteUnavailableError = class extends Error {
};
var publicKey = PLazy.from(async function loadPublicKey(retry = 0) {
  try {
    const res = await fetch(cloudConfig.jwks);
    if (res.status !== 200)
      throw new HttpError(res.status, await res.text());
    const result = await res.json();
    const jwks = result;
    const key = jwks.keys[0];
    if (!key)
      throw new HttpError(500, "No signature key found");
    return key;
  } catch (error) {
    if (retry < 3)
      return loadPublicKey(retry + 1);
    publicKey = PLazy.from(loadPublicKey);
    throw new RemoteUnavailableError("Remote unavailable", { cause: error });
  }
});
var COOKIE_NAME = "alinea.cloud";
function cloudBackend(config) {
  const auth = {
    async authenticate(ctx, request) {
      const url = new URL(request.url);
      const action = url.searchParams.get("auth");
      let dashboardPath = config.dashboardFile ?? "/admin.html";
      if (!dashboardPath.startsWith("/"))
        dashboardPath = "/" + dashboardPath;
      const dashboardUrl = new URL(dashboardPath, url);
      switch (action) {
        case AuthAction.Status: {
          const token = ctx.apiKey && ctx.apiKey.split("_")[1];
          if (!token)
            return Response.json({
              type: AuthResultType.MissingApiKey,
              setupUrl: cloudConfig.setup
            });
          const [authed, err] = await outcome(this.verify(ctx, request));
          if (authed)
            return Response.json({
              type: AuthResultType.Authenticated,
              user: authed.user
            });
          return Response.json({
            type: AuthResultType.UnAuthenticated,
            redirect: `${cloudConfig.auth}?token=${token}`
          });
        }
        case AuthAction.Handshake: {
          const handShakeId = url.searchParams.get("handshake_id");
          if (!handShakeId)
            throw new HttpError(
              400,
              "Provide a valid handshake id to initiate handshake"
            );
          const body = {
            handshake_id: handShakeId,
            status: {
              version: package_default.version,
              roles: [
                {
                  key: "editor",
                  label: "Editor",
                  description: "Can view and edit all pages"
                }
              ],
              sourceDirectories: Object.values(config.workspaces).flatMap((workspace) => {
                const { source, mediaDir } = Workspace.data(workspace);
                return [source, mediaDir];
              }).filter(Boolean)
            }
          };
          const res = await fetch(cloudConfig.handshake, {
            method: "POST",
            body: JSON.stringify(body),
            headers: {
              "content-type": "application/json",
              accept: "application/json",
              ...bearer(ctx)
            }
          }).catch((e) => {
            throw new HttpError(500, `Could not reach handshake api: ${e}`);
          });
          if (res.status !== 200)
            throw new HttpError(
              res.status,
              `Handshake failed: ${await res.text()}`
            );
          return new Response("alinea cloud handshake");
        }
        case AuthAction.Login: {
          const token = url.searchParams.get("token");
          if (!token)
            throw new HttpError(400, "Token required");
          const user = await verify(token, await publicKey);
          return router.redirect(dashboardUrl.href, {
            status: 302,
            headers: {
              "set-cookie": router.cookie({
                name: COOKIE_NAME,
                value: token,
                domain: dashboardUrl.hostname,
                path: "/",
                secure: dashboardUrl.protocol === "https:",
                httpOnly: true,
                sameSite: "strict"
              })
            }
          });
        }
        case AuthAction.Logout: {
          try {
            const { token } = await this.verify(ctx, request);
            if (token) {
              await fetch(cloudConfig.logout, {
                method: "POST",
                headers: { authorization: `Bearer ${token}` }
              });
            }
          } catch (e) {
            console.error(e);
          }
          return router.redirect(dashboardUrl.href, {
            status: 302,
            headers: {
              "set-cookie": router.cookie({
                name: COOKIE_NAME,
                value: "",
                domain: dashboardUrl.hostname,
                path: "/",
                secure: dashboardUrl.protocol === "https:",
                httpOnly: true,
                sameSite: "strict",
                expires: /* @__PURE__ */ new Date(0)
              })
            }
          });
        }
        default:
          return new Response("Bad request", { status: 400 });
      }
    },
    async verify(ctx, request) {
      const cookies = request.headers.get("cookie");
      if (!cookies)
        throw new HttpError(401, "Unauthorized - no cookies");
      const prefix = `${COOKIE_NAME}=`;
      const token = cookies.split(";").map((c) => c.trim()).find((c) => c.startsWith(prefix));
      if (!token)
        throw new HttpError(401, `Unauthorized - no ${COOKIE_NAME}`);
      const authToken = token.slice(prefix.length);
      return {
        ...ctx,
        token: authToken,
        user: await verify(authToken, await publicKey)
      };
    }
  };
  const target = {
    async mutate(ctx, params) {
      return parseOutcome(
        fetch(
          cloudConfig.mutate,
          json({
            method: "POST",
            headers: bearer(ctx),
            body: JSON.stringify(params)
          })
        )
      );
    }
  };
  const media = {
    prepareUpload(ctx, file) {
      return parseOutcome(
        fetch(
          cloudConfig.upload,
          json({
            method: "POST",
            headers: bearer(ctx),
            body: JSON.stringify({ filename: file })
          })
        )
      ).then(({ upload, ...rest }) => {
        return {
          ...rest,
          method: upload.method,
          url: upload.url
        };
      });
    }
  };
  const drafts = {
    async get(ctx, entryId) {
      if (!validApiKey(ctx.apiKey))
        return;
      const data = await parseOutcome(
        fetch(cloudConfig.drafts + "/" + entryId, json({ headers: bearer(ctx) }))
      );
      return data?.update ? {
        entryId,
        commitHash: data.commitHash,
        fileHash: data.fileHash,
        draft: base64.parse(data.update)
      } : void 0;
    },
    store(ctx, draft) {
      return parseOutcome(
        fetch(
          cloudConfig.drafts + "/" + draft.entryId,
          json({
            method: "PUT",
            headers: bearer(ctx),
            body: JSON.stringify({
              fileHash: draft.fileHash,
              update: base64.stringify(draft.draft)
            })
          })
        )
      );
    }
  };
  const history = {
    async list(ctx, file) {
      return parseOutcome(
        fetch(
          cloudConfig.history + "?" + new URLSearchParams({ file }),
          json({ headers: bearer(ctx) })
        )
      );
    },
    async revision(ctx, file, ref) {
      return parseOutcome(
        fetch(
          cloudConfig.history + "?" + new URLSearchParams({ file, ref }),
          json({ headers: bearer(ctx) })
        )
      );
    }
  };
  const pending = {
    async since(ctx, commitHash) {
      if (!validApiKey(ctx.apiKey))
        return;
      return parseOutcome(
        fetch(
          cloudConfig.pending + "?" + new URLSearchParams({ since: commitHash }),
          json({ headers: bearer(ctx) })
        )
      ).then((pending2) => {
        if (pending2.length === 0)
          return void 0;
        return {
          toCommitHash: pending2[pending2.length - 1].commitHashTo,
          mutations: pending2.flatMap(
            (mutate) => mutate.mutations.flatMap((m) => m.meta)
          )
        };
      });
    }
  };
  return {
    auth,
    target,
    media,
    drafts,
    history,
    pending
  };
}
function validApiKey(apiKey) {
  return Boolean(apiKey?.startsWith("alineapk"));
}
function bearer(ctx) {
  return { authorization: `Bearer ${"token" in ctx ? ctx.token : ctx.apiKey}` };
}
function json(init = {}) {
  const headers = new Headers(init.headers);
  if (init.body)
    headers.set("content-type", "application/json");
  headers.set("accept", "application/json");
  return { ...init, headers };
}
async function failOnHttpError(res) {
  if (res.status >= 400)
    throw new HttpError(res.status, await res.text());
  return res;
}
function parseJson(res) {
  return res.json();
}
function parseOutcome(res) {
  return res.then(failOnHttpError).then(parseJson).then(Outcome.fromJSON).then(Outcome.unpack);
}
export {
  cloudBackend
};
