import "../chunks/chunk-U5RRZUYZ.js";

// src/core/Client.ts
import { AbortController, fetch } from "@alinea/iso";
import { HandleAction } from "alinea/backend/HandleAction";
import { HttpError } from "./HttpError.js";
import { base64 } from "./util/Encoding.js";
var Client = class _Client {
  #options;
  constructor(options) {
    this.#options = options;
  }
  get url() {
    return this.#options.url;
  }
  previewToken(request) {
    return this.#requestJson(
      { action: HandleAction.PreviewToken },
      {
        method: "POST",
        body: JSON.stringify(request)
      }
    ).then(this.#failOnHttpError);
  }
  prepareUpload(file) {
    return this.#requestJson(
      { action: HandleAction.Upload },
      {
        method: "POST",
        body: JSON.stringify({ filename: file })
      }
    ).then(this.#failOnHttpError);
  }
  user() {
    return this.#requestJson({ action: HandleAction.User }).then(this.#failOnHttpError).then((user) => user ?? void 0);
  }
  resolve(params) {
    const body = JSON.stringify(params);
    return this.#requestJson(
      { action: HandleAction.Resolve },
      { method: "POST", body }
    ).then(this.#failOnHttpError);
  }
  mutate(mutations) {
    return this.#requestJson(
      { action: HandleAction.Mutate },
      { method: "POST", body: JSON.stringify(mutations) }
    ).then(this.#failOnHttpError);
  }
  authenticate(applyAuth, unauthorized) {
    return new _Client({ ...this.#options, applyAuth, unauthorized });
  }
  // History
  revisions(file) {
    return this.#requestJson({ action: HandleAction.History, file }).then(this.#failOnHttpError);
  }
  revisionData(file, revisionId) {
    return this.#requestJson({
      action: HandleAction.History,
      file,
      revisionId
    }).then(this.#failOnHttpError).then((res) => res ?? void 0);
  }
  // Syncable
  syncRequired(contentHash) {
    return this.#requestJson({
      action: HandleAction.Sync,
      contentHash
    }).then(this.#failOnHttpError);
  }
  sync(contentHashes) {
    return this.#requestJson(
      { action: HandleAction.Sync },
      { method: "POST", body: JSON.stringify(contentHashes) }
    ).then(this.#failOnHttpError);
  }
  // Drafts
  getDraft(entryId) {
    return this.#requestJson({ action: HandleAction.Draft, entryId }).then(this.#failOnHttpError).then(
      (draft) => draft ? { ...draft, draft: base64.parse(draft.draft) } : void 0
    );
  }
  storeDraft(draft) {
    return this.#requestJson(
      { action: HandleAction.Draft },
      {
        method: "POST",
        body: JSON.stringify({ ...draft, draft: base64.stringify(draft.draft) })
      }
    ).then((res) => this.#failOnHttpError(res, false));
  }
  #request(params, init) {
    const { url, applyAuth = (v) => v, unauthorized } = this.#options;
    const controller = new AbortController();
    const signal = controller.signal;
    const location = url + "?" + new URLSearchParams(params).toString();
    const promise = fetch(location, {
      ...applyAuth(init),
      signal
    }).catch((err) => {
      throw new HttpError(
        500,
        `Could not ${init?.method || "GET"} "${params.action}": ${err}`
      );
    }).then(async (res) => {
      if (res.status === 401)
        unauthorized?.();
      if (!res.ok) {
        const isJson = res.headers.get("content-type")?.includes("application/json");
        const msg = isJson ? JSON.stringify(await res.json(), null, 2) : await res.text();
        throw new HttpError(
          res.status,
          `Could not ${init?.method || "GET"} "${params.action}" (${res.status}) ... ${msg.replace(/\s+/g, " ").slice(0, 100)}`
        );
      }
      return res;
    });
    const cancel = () => controller.abort();
    function cancelify(promise2) {
      const t = promise2.then.bind(promise2);
      const c = promise2.catch.bind(promise2);
      return Object.assign(promise2, {
        cancel,
        then: (...args) => cancelify(t(...args)),
        catch: (...args) => cancelify(c(...args))
      });
    }
    return cancelify(promise);
  }
  #requestJson(params, init) {
    return this.#request(params, {
      ...init,
      headers: {
        ...init?.headers,
        "content-type": "application/json",
        accept: "application/json"
      }
    });
  }
  async #failOnHttpError(res, expectJson = true) {
    if (res.ok)
      return expectJson ? res.json() : void 0;
    const text = await res.text();
    throw new HttpError(res.status, text || res.statusText);
  }
};
export {
  Client
};
