import "../chunks/chunk-U5RRZUYZ.js";

// src/core/Transaction.ts
import { Blob } from "@alinea/iso";
import { Entry } from "./Entry.js";
import { EntryPhase } from "./EntryRow.js";
import { HttpError } from "./HttpError.js";
import { createId } from "./Id.js";
import { MutationType } from "./Mutation.js";
import { Root } from "./Root.js";
import { Schema } from "./Schema.js";
import { Type } from "./Type.js";
import { Workspace } from "./Workspace.js";
import { isImage } from "./media/IsImage.js";
import { createFileHash } from "./util/ContentHash.js";
import {
  entryChildrenDir,
  entryFileName,
  entryFilepath,
  entryUrl,
  workspaceMediaDir
} from "./util/EntryFilenames.js";
import { createEntryRow, entryParentPaths } from "./util/EntryRows.js";
import { generateKeyBetween } from "./util/FractionalIndexing.js";
import { basename, extname, join, normalize } from "./util/Paths.js";
import { slugify } from "./util/Slugs.js";
var Operation = class _Operation {
  static Data = Symbol.for("@alinea/Operation.Data");
  constructor(tx) {
    this[_Operation.Data] = tx;
  }
  typeName(config, type) {
    const typeNames = Schema.typeNames(config.schema);
    const typeName = typeNames.get(type);
    if (!typeName)
      throw new Error(`Type "${Type.label(type)}" not found in Schema`);
    return typeName;
  }
};
var UploadOperation = class extends Operation {
  entryId = createId();
  parentId;
  workspace;
  root;
  constructor(file, options = {}) {
    super(async ({ config, graph, connect }) => {
      const fileName = Array.isArray(file) ? file[0] : file.name;
      const body = Array.isArray(file) ? file[1] : await file.arrayBuffer();
      const workspace = this.workspace ?? Object.keys(config.workspaces)[0];
      const root = this.root ?? Workspace.defaultMediaRoot(config.workspaces[workspace]);
      const extension = extname(fileName);
      const path = slugify(basename(fileName, extension));
      const directory = workspaceMediaDir(config, workspace);
      const uploadLocation = join(directory, path + extension);
      const cnx = await connect();
      const info = await cnx.prepareUpload(uploadLocation);
      const previewData = isImage(fileName) ? await options.createPreview?.(
        file instanceof Blob ? file : new Blob([body])
      ) : void 0;
      fetch(info.url, { method: info.method ?? "POST", body }).then(
        async (result) => {
          if (!result.ok)
            throw new HttpError(
              result.status,
              `Could not reach server for upload`
            );
        }
      );
      const parent = this.parentId ? await graph.preferPublished.get(Entry({ entryId: this.parentId })) : void 0;
      const title = basename(fileName, extension);
      const hash = await createFileHash(new Uint8Array(body));
      const { mediaDir } = Workspace.data(config.workspaces[workspace]);
      const prefix = mediaDir && normalize(mediaDir);
      const fileLocation = prefix && info.location.startsWith(prefix) ? info.location.slice(prefix.length) : info.location;
      const entryData = {
        title,
        location: fileLocation,
        extension,
        size: body.byteLength,
        hash,
        ...previewData
      };
      const entry = await createEntry(
        config,
        "MediaFile",
        {
          path,
          entryId: this.entryId,
          workspace,
          root,
          data: entryData
        },
        parent
      );
      const parentPaths = entryParentPaths(config, entry);
      const entryFile = entryFileName(config, entry, parentPaths);
      return [
        {
          type: MutationType.Upload,
          entryId: this.entryId,
          url: info.previewUrl,
          file: info.location
        },
        {
          type: MutationType.Create,
          entryId: entry.entryId,
          file: entryFile,
          entry
        }
      ];
    });
  }
  setParent(parentId) {
    this.parentId = parentId;
    return this;
  }
  setWorkspace(workspace) {
    this.workspace = workspace;
    return this;
  }
  setRoot(root) {
    this.root = root;
    return this;
  }
};
var DeleteOp = class extends Operation {
  constructor(entryId) {
    super(async ({ config, graph }) => {
      const entry = await graph.preferPublished.get(
        Entry({ entryId: this.entryId })
      );
      const parentPaths = entryParentPaths(config, entry);
      const file = entryFileName(config, entry, parentPaths);
      return [
        {
          type: MutationType.Remove,
          entryId: this.entryId,
          file
        }
      ];
    });
    this.entryId = entryId;
  }
};
var EditOperation = class extends Operation {
  constructor(entryId) {
    super(async ({ config, graph }) => {
      let realm;
      if (this.changePhase === EntryPhase.Draft)
        realm = graph.preferDraft;
      else if (this.changePhase === EntryPhase.Archived)
        realm = graph.preferPublished;
      else if (this.changePhase === EntryPhase.Published)
        realm = graph.preferDraft;
      else
        realm = graph.preferPublished;
      const entry = await realm.get(Entry({ entryId: this.entryId }));
      const parent = entry.parent ? await graph.preferPublished.get(Entry({ entryId: entry.parent })) : void 0;
      const parentPaths = entryParentPaths(config, entry);
      const file = entryFileName(
        config,
        { ...entry, phase: entry.phase },
        parentPaths
      );
      const type = config.schema[entry.type];
      const mutations = [];
      const createDraft = this.changePhase === EntryPhase.Draft;
      if (createDraft)
        mutations.push({
          type: MutationType.Edit,
          entryId: this.entryId,
          file,
          entry: await createEntry(
            config,
            this.typeName(config, type),
            {
              ...entry,
              phase: EntryPhase.Draft,
              data: { ...entry.data, ...this.entryData }
            },
            parent
          )
        });
      else if (this.entryData)
        mutations.push({
          type: MutationType.Patch,
          entryId: this.entryId,
          file,
          patch: this.entryData
        });
      switch (this.changePhase) {
        case EntryPhase.Published:
          mutations.push({
            type: MutationType.Publish,
            phase: entry.phase,
            entryId: this.entryId,
            file
          });
          break;
        case EntryPhase.Archived:
          mutations.push({
            type: MutationType.Archive,
            entryId: this.entryId,
            file
          });
          break;
      }
      return mutations;
    });
    this.entryId = entryId;
  }
  entryData;
  changePhase;
  set(entryData) {
    this.entryData = { ...this.entryData, ...entryData };
    return this;
  }
  draft() {
    this.changePhase = EntryPhase.Draft;
    return this;
  }
  archive() {
    this.changePhase = EntryPhase.Archived;
    return this;
  }
  publish() {
    this.changePhase = EntryPhase.Published;
    return this;
  }
};
var CreateOperation = class _CreateOperation extends Operation {
  constructor(entry, type, parentRow) {
    super(async (cms) => {
      const entry2 = await this.entryRow(cms);
      const parentPaths = entryParentPaths(cms.config, entry2);
      const file = entryFileName(cms.config, entry2, parentPaths);
      return [
        {
          type: MutationType.Create,
          entryId: entry2.entryId,
          file,
          entry: entry2
        }
      ];
    });
    this.type = type;
    this.parentRow = parentRow;
    this.entry = { entryId: createId(), ...entry };
  }
  /** @internal */
  entry;
  entryRow = async (cms) => {
    const partial = this.entry;
    const type = this.type ? this.typeName(cms.config, this.type) : partial.type;
    if (!type)
      throw new TypeError(`Type is missing`);
    const parent = await (this.parentRow ? this.parentRow(cms) : partial.parent ? cms.graph.preferPublished.get(Entry({ entryId: partial.parent })) : void 0);
    const previousIndex = await cms.graph.preferPublished.maybeGet(
      Entry({ parent: parent?.entryId ?? null }).select(Entry.index).orderBy(Entry.index.desc())
    );
    const index = generateKeyBetween(previousIndex, null);
    return createEntry(cms.config, type, { ...partial, index }, parent);
  };
  setParent(parentId) {
    this.entry.parent = parentId;
    return this;
  }
  setWorkspace(workspace) {
    this.entry.workspace = workspace;
    return this;
  }
  setRoot(root) {
    this.entry.root = root;
    return this;
  }
  setLocale(locale) {
    this.entry.locale = locale;
    return this;
  }
  set(entryData) {
    this.entry.data = { ...this.entry.data, ...entryData };
    return this;
  }
  createChild(type) {
    return new _CreateOperation({}, type, this.entryRow);
  }
  get entryId() {
    return this.entry.entryId;
  }
  static fromType(type) {
    return new _CreateOperation({}, type);
  }
};
async function createEntry(config, typeName, partial = { title: "Entry" }, parent) {
  const type = config.schema[typeName];
  const workspace = parent?.workspace ?? partial.workspace ?? Object.keys(config.workspaces)[0];
  const root = parent?.root ?? partial.root ?? Object.keys(config.workspaces[workspace])[0];
  const locale = parent?.locale ?? partial.locale ?? Root.defaultLocale(config.workspaces[workspace][root]) ?? null;
  const title = partial.data?.title ?? partial.title ?? "Entry";
  const phase = partial.phase ?? EntryPhase.Published;
  const path = slugify(
    phase === EntryPhase.Published && partial.data?.path || (partial.path ?? title)
  );
  const entryData = { title, path, ...partial.data };
  const entryId = partial.entryId ?? createId();
  const i18nId = partial.i18nId ?? createId();
  const details = {
    entryId,
    phase,
    type: typeName,
    title,
    path,
    seeded: null,
    workspace,
    root,
    level: parent ? parent.level + 1 : 0,
    parent: parent?.entryId ?? null,
    locale,
    index: partial.index ?? "a0",
    i18nId,
    modifiedAt: 0,
    active: partial.active ?? true,
    main: partial.main ?? true,
    data: entryData,
    searchableText: Type.searchableText(type, entryData)
  };
  const isI18n = Boolean(Root.data(config.workspaces[workspace][root]).i18n);
  const parentPaths = parent?.childrenDir.split("/").filter(Boolean).slice(isI18n ? 1 : 0) ?? [];
  const filePath = entryFilepath(config, details, parentPaths);
  const childrenDir = entryChildrenDir(config, details, parentPaths);
  const urlMeta = {
    locale,
    path,
    phase,
    parentPaths
  };
  const url = entryUrl(type, urlMeta);
  return createEntryRow(config, {
    ...details,
    filePath,
    childrenDir,
    parentDir: childrenDir.split("/").slice(0, -1).join("/"),
    url
  });
}
export {
  CreateOperation,
  DeleteOp,
  EditOperation,
  Operation,
  UploadOperation
};
