import "../../chunks/chunk-U5RRZUYZ.js";

// src/core/pages/Expr.ts
import { entries, fromEntries } from "../util/Objects.js";
import {
  BinaryOp,
  ExprData,
  OrderDirection,
  Selection,
  UnaryOp,
  toExpr,
  toSelection
} from "./ResolveData.js";
function Expr(expr) {
  return new ExprI(expr);
}
var ExprI = class {
  constructor(expr) {
    this[Expr.Data] = expr;
    this[Expr.ExprRef] = true;
  }
  asc() {
    return { expr: this[Expr.Data], order: OrderDirection.Asc };
  }
  desc() {
    return { expr: this[Expr.Data], order: OrderDirection.Desc };
  }
  not() {
    return Expr(ExprData.UnOp(UnaryOp.Not, this[Expr.Data]));
  }
  or(...that) {
    let res = this;
    if (this.isConstant(true))
      return res;
    for (const e of that) {
      const expr = Expr.create(e);
      if (expr.isConstant(true))
        return expr;
      if (expr.isConstant(false))
        continue;
      res = Expr(ExprData.BinOp(res[Expr.Data], BinaryOp.Or, expr[Expr.Data]));
    }
    return res;
  }
  and(...that) {
    let res = this;
    if (this.isConstant(false))
      return res;
    for (const e of that) {
      const expr = Expr.create(e);
      if (expr.isConstant(false))
        return expr;
      if (expr.isConstant(true))
        continue;
      res = Expr(ExprData.BinOp(res[Expr.Data], BinaryOp.And, expr[Expr.Data]));
    }
    return res;
  }
  is(that) {
    if (that === null || Expr.isExpr(that) && that.isConstant(null))
      return this.isNull();
    return Expr(
      ExprData.BinOp(this[Expr.Data], BinaryOp.Equals, createExprData(that))
    );
  }
  isConstant(value) {
    const expr = this[Expr.Data];
    switch (expr.type) {
      case "value":
        return expr.value === value;
      default:
        return false;
    }
  }
  isNot(that) {
    if (that === null || Expr.isExpr(that) && that.isConstant(null))
      return this.isNotNull();
    return Expr(
      ExprData.BinOp(this[Expr.Data], BinaryOp.NotEquals, createExprData(that))
    );
  }
  isNull() {
    return Expr(ExprData.UnOp(UnaryOp.IsNull, this[Expr.Data]));
  }
  isNotNull() {
    return this.isNull().not();
  }
  isIn(that) {
    return Expr(
      ExprData.BinOp(this[Expr.Data], BinaryOp.In, createExprData(that))
    );
  }
  isNotIn(that) {
    return Expr(
      ExprData.BinOp(this[Expr.Data], BinaryOp.NotIn, createExprData(that))
    );
  }
  isBetween(min, max) {
    return this.isGreaterOrEqual(min).and(this.isLessOrEqual(max));
  }
  isGreater(that) {
    return Expr(
      ExprData.BinOp(this[Expr.Data], BinaryOp.Greater, createExprData(that))
    );
  }
  isGreaterOrEqual(that) {
    return Expr(
      ExprData.BinOp(
        this[Expr.Data],
        BinaryOp.GreaterOrEqual,
        createExprData(that)
      )
    );
  }
  isLess(that) {
    return Expr(
      ExprData.BinOp(this[Expr.Data], BinaryOp.Less, createExprData(that))
    );
  }
  isLessOrEqual(that) {
    return Expr(
      ExprData.BinOp(
        this[Expr.Data],
        BinaryOp.LessOrEqual,
        createExprData(that)
      )
    );
  }
  add(...args) {
    return Expr(
      args.map(Expr.create).map((expr) => expr[Expr.Data]).reduce((a, b) => ExprData.BinOp(a, BinaryOp.Add, b))
    );
  }
  subtract(...args) {
    return Expr(
      args.map(Expr.create).map((expr) => expr[Expr.Data]).reduce((a, b) => ExprData.BinOp(a, BinaryOp.Subt, b))
    );
  }
  multiply(...args) {
    return Expr(
      args.map(Expr.create).map((expr) => expr[Expr.Data]).reduce((a, b) => ExprData.BinOp(a, BinaryOp.Mult, b))
    );
  }
  remainder(that) {
    return Expr(
      ExprData.BinOp(this[Expr.Data], BinaryOp.Mod, createExprData(that))
    );
  }
  divide(...args) {
    return Expr(
      args.map(Expr.create).map((expr) => expr[Expr.Data]).reduce((a, b) => ExprData.BinOp(a, BinaryOp.Div, b))
    );
  }
  concat(that) {
    return Expr(
      ExprData.BinOp(this[Expr.Data], BinaryOp.Concat, createExprData(that))
    );
  }
  endsWith(that) {
    return Expr(
      ExprData.BinOp(this[Expr.Data], BinaryOp.Like, createExprData(`%${that}`))
    );
  }
  startsWith(that) {
    return Expr(
      ExprData.BinOp(this[Expr.Data], BinaryOp.Like, createExprData(`${that}%`))
    );
  }
  like(that) {
    return Expr(
      ExprData.BinOp(this[Expr.Data], BinaryOp.Like, createExprData(that))
    );
  }
  /*dynamic<X = T>(...path: Array<string>): Fields<X> {
    return new Proxy<any>(
      (...args: Array<any>) => {
        const method = path[path.length - 1]
        const e: any =
          path.length > 1 ? this.get(path.slice(0, -1).join('.')) : this
        return e[method]?.apply(e, args)
      },
      {
        get: (_, key) => {
          const e: any = path.length > 0 ? this.get(path.join('.')) : this
          if (typeof key !== 'string') return e[key]
          return this.dynamic(...path, key)
        }
      }
    )
  }*/
  at(index) {
    return this.get(`[${Number(index)}]`);
  }
  includes(value) {
    return Expr(createExprData(value)).isIn(this);
  }
  sure() {
    return this;
  }
  get(name) {
    return Expr(ExprData.Access(this[Expr.Data], name));
  }
  toJSON() {
    return this[Expr.Data];
  }
  [toSelection]() {
    return Selection.Expr(this[Expr.Data]);
  }
};
((Expr2) => {
  Expr2.Data = Symbol.for("@alinea/Expr.Data");
  Expr2.ExprRef = Symbol.for("@alinea/Expr.ExprRef");
  Expr2.NULL = create(null);
  function create(input) {
    if (hasExpr(input))
      return input[toExpr]();
    if (isExpr(input))
      return input;
    return Expr2(ExprData.Value(input));
  }
  Expr2.create = create;
  function hasExpr(input) {
    return input && (typeof input === "function" || typeof input === "object") && input[toExpr];
  }
  Expr2.hasExpr = hasExpr;
  function isExpr(input) {
    return input !== null && (typeof input === "object" || typeof input === "function") && input[Expr2.ExprRef];
  }
  Expr2.isExpr = isExpr;
  function and(...conditions) {
    return conditions.map(Expr2.create).reduce(
      (condition, expr) => condition.and(expr),
      Expr2(ExprData.Value(true))
    );
  }
  Expr2.and = and;
  function or(...conditions) {
    return conditions.map(Expr2.create).reduce(
      (condition, expr) => condition.or(expr),
      Expr2(ExprData.Value(false))
    );
  }
  Expr2.or = or;
})(Expr || (Expr = {}));
function createExprData(input) {
  if (input === null || input === void 0)
    return ExprData.Value(null);
  if (Expr.hasExpr(input))
    input = input[toExpr]();
  if (Expr.isExpr(input))
    return input[Expr.Data];
  if (input && typeof input === "object" && !Array.isArray(input))
    return ExprData.Record(
      fromEntries(
        entries(input).map(([key, value]) => [key, createExprData(value)])
      )
    );
  return ExprData.Value(input);
}
export {
  Expr,
  ExprI,
  createExprData
};
